# (c) Copyright 2018 Trent Hauck
# All Rights Reserved

from typing import NamedTuple

import tensorflow as tf
import numpy as np


def to_tensorflow_record(encoded_seq) -> tf.train.Example:
    """
    Convert the sequence to a tensorflow record.
    """

    integer_encoded = tf.train.Feature(
        int64_list=tf.train.Int64List(value=encoded_seq.integer_encoded)
    )

    alphabet_letters = tf.train.Feature(
        bytes_list=tf.train.BytesList(
            value=[m.encode("utf-8") for m in encoded_seq.alphabet.letters_and_tokens]
        )
    )

    feature_arg = {"integer_encoded": integer_encoded, "alphabet_letters": alphabet_letters}
    example = tf.train.Example(features=tf.train.Features(feature=feature_arg))

    return example


# TODO: Should this be SequenceRecordExample?? I.e. probably want to have the actual id of the
# sequence for later use.
class ParsedEncodedSequence(NamedTuple):
    integer_encoded: np.ndarray
    alphabet_letters: np.ndarray


def from_tensorflow_example(example: tf.train.Example) -> ParsedEncodedSequence:
    """
    Given the example return an encoded sequence with the integer encoding and alphabet.
    """

    features = {
        "integer_encoded": tf.FixedLenSequenceFeature((), tf.int64, allow_missing=True),
        "alphabet_letters": tf.FixedLenSequenceFeature((), tf.string, allow_missing=True),
    }
    parsed_features = tf.parse_single_example(example, features)
    return ParsedEncodedSequence(
        integer_encoded=parsed_features["integer_encoded"],
        alphabet_letters=parsed_features["alphabet_letters"],
    )



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['gcgc',
 'gcgc.alphabet',
 'gcgc.encoded_seq',
 'gcgc.tests',
 'gcgc.tests.alphabet',
 'gcgc.tests.cli',
 'gcgc.tests.encoded_seq',
 'gcgc.tests.fixtures',
 'gcgc.tests.third_party.tensorflow_utils',
 'gcgc.third_party',
 'gcgc.third_party.tensorflow_utils']

package_data = \
{'': ['*'], 'gcgc.tests.fixtures': ['p53_human/*']}

install_requires = \
['biopython==1.72', 'numpy==1.15.2', 'tensorflow>=1.12,<2.0']

entry_points = \
{'console_scripts': ['gcgc = gcgc.cli:main']}

setup_kwargs = {
    'name': 'gcgc',
    'version': '0.2.2',
    'description': 'GCGC is a preprocessing library for biological sequence model development.',
    'long_description': None,
    'author': 'Trent Hauck',
    'author_email': 'trent@trenthauck.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# (c) Copyright 2018 Trent Hauck
# All Rights Reserved

import unittest

from Bio import SeqIO

import tensorflow as tf

from gcgc.encoded_seq import EncodedSeq
from gcgc.third_party.tensorflow_utils import record as gcgc_record
from gcgc.tests.fixtures import P53_HUMAN


class TestTFRecordSerialization(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_read_tf_records(self):

        with open(P53_HUMAN) as f:
            sr = SeqIO.read(f, "fasta")

        encoded_seq = EncodedSeq.from_seq(sr.seq)
        filenames = [str(P53_HUMAN.with_suffix(".tf_records"))]

        dataset = tf.data.TFRecordDataset(filenames)
        dataset = dataset.map(gcgc_record.from_tensorflow_example)
        dataset_iterator = dataset.make_one_shot_iterator()
        next_item = dataset_iterator.get_next()

        session = tf.Session()
        seq = session.run(next_item)

        self.assertEqual(encoded_seq.integer_encoded, seq.integer_encoded.tolist())



# (c) Copyright 2018 Trent Hauck
# All Rights Reserved

import sys
import click
import pathlib
import logging

from Bio import SeqIO
import tensorflow as tf

from gcgc import __version__
from gcgc.encoded_seq import EncodedSeq
from gcgc.third_party.tensorflow_utils import record

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


@click.group()
def main(args=None):
    """Console script for gcgc."""
    return 0


@main.command()
def version():
    print(__version__)


def to_path(ctx, param, value) -> pathlib.Path:
    """
    Simple callback to convert the string in the option to a command.
    """

    return pathlib.Path(value)


@main.command()
@click.argument("filename", callback=to_path)
@click.argument("format")
@click.option("-e", "--encapsulate", is_flag=True)
@click.option("-c", "--conform-to-length", "conform_to", type=int)
@click.option("-p", "--pad-to-length", "pad_to", type=int)
def convert_file_to_tf_records(filename, format, encapsulate, conform_to, pad_to):

    output_file = filename.with_suffix(".tf-records")
    logger.info(f"Reading from {filename} in format {format} and writing to {output_file}.")

    writer = tf.python_io.TFRecordWriter(str(output_file))
    try:
        with open(filename, "rU") as handle:
            for seq_record in SeqIO.parse(handle, format):

                encoded_seq = EncodedSeq.from_seq(seq_record.seq)

                if encapsulate:
                    encoded_seq = encoded_seq.encapsulate()

                if conform_to:
                    encoded_seq = encoded_seq.conform(conform_to)

                if pad_to:
                    encoded_seq = encoded_seq.pad(pad_to)

                example = record.to_tensorflow_record(encoded_seq)
                writer.write(example.SerializeToString())
    finally:
        writer.close()


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover
